%% Kyoto University, Statistical Machine Learning - Lecture 1 scripts and data
% Marco Cuturi (c) 2011


%% This loads the dataset
load HOUSING;

% scatter plot of rent vs surface. The 4th column is divided by 10.000 to
% have rent in MAN YENs.
scatter(HOUSING(:,2),HOUSING(:,4)/10000,20,'rs','filled');
title('Scatter plot of Rent vs. Surface');
xlabel('Surface')
ylabel('Rent (x 10.000 JPY)')
xlim([10 80]);
ylim([2 14]);

% Reshuffle entries. Otherwise they are somehow ordered by rent price.
P=randperm(285);
H=HOUSING(P,:);

% Plot the matrix using rent in 1000 JPY (looks better)
figure;
H(:,4)=H(:,4)/1000;
imagesc(H(P,:)');
set(gca,'YTick',1:4);
set(gca,'YTickLabel',{'Age','Surface','Dist','Rent x 1.000'});
colorbar
set(gca,'FontSize',20);

%% Contour plot of least-squares (LINEAR)
figure;
H(:,4)=H(:,4)/10; % to get rent in MAN YEN
g= @(a,b) norm(H(:,4)-(a*H(:,2)+b))^2; % this defines a function handle
ezcontourf(g,[.11,.19,1.5,3],100); % ezcontour can plot directly function handles.


%% least square coefficients recovered analytically (LINEAR)
X=H(:,2);
Y=H(:,4);

T=[sum(X.^2),sum(X);sum(X),length(X)];
T\[sum(X.*Y);sum(Y)] % Here you get again a and b

%% least square coefficients recovered analytically (QUADRATIC)
T2=[sum(X.^4), sum(X.^3), sum(X.^2); sum(X.^3), sum(X.^2), sum(X);sum(X.^2),sum(X),length(X)];
T2\[sum((X.^2).*Y); sum(X.*Y);sum(Y)]


%% This produces the figure of the "General Case" slide.
figure;
subplot(3,1,1)
imagesc(H(:,4)')
set(gca,'YTick',1);
set(gca,'YTickLabel',{'Rent x 10.000'});
colorbar
set(gca,'FontSize',20);

subplot(3,1,2:3)
imagesc(H(P,1:3)');
set(gca,'YTick',1:4);
set(gca,'YTickLabel',{'Age','Surface','Dist'});
colorbar
set(gca,'FontSize',20);


%% Least-square coefficients using all 3 variables + constant
X=[H(P,1:3)';ones(1,285)];
Y=H(:,4)';
(X*X')\(X*Y')

%% Removing outliers (only keep surface < 40
H=H(H(:,2)<40,:);

% This is what the scatter plot now looks like
figure
scatter(H(:,2),H(:,4),20,'rs','filled');
title('Scatter plot of Rent vs. Surface');
xlabel('Surface')
ylabel('Rent (x 10.000 JPY)')
xlim([10 50]);
ylim([2 9]);

% And here are the new coefficients.
X=[H(:,1:3)';ones(1,size(H,1))];
Y=H(:,4)';
inv(X*X')*(X*Y')
conditionnumber = cond(X*X')


%% Compute and plot residues
figure();
res=Y'-X'*((X*X')\(X*Y'));
sigma=std(res);
hold on
[n,xout] = hist(res,30); %Use 30 bins for the histogram
bar(xout,n/sum(n)); %relative frequency is n/sum(n)
ezplot(['.1*exp(-x^2/(2*',num2str(sigma^2),'))'],[min(xout),max(xout)]);
hold off


% Add measurements in ?, look at condition number
H(:,5)=H(:,2)/1.4+.001*randn(size(H,1),1); % Creating the measurement in tatami mats.
X=[H(:,[1:3,5])';ones(1,size(H,1))];
Y=H(:,4)';

conditionnumber = cond(X*X')
inv(X*X')*(X*Y')

% See the effect of just removing one observation
Xminusoneobs=X(:,1:end-1);
Yminusoneobs=Y(:,1:end-1);

inv(Xminusoneobs*Xminusoneobs')*(Xminusoneobs*Yminusoneobs')

%% Adding a noise variable
X=[H(:,1:3)';ones(1,size(H,1))];
res=Y'-X'*((X*X')\(X*Y'));
RSS=sum(res.^2)

X=[H(:,1:3)';ones(1,size(H,1))];
X=[X;rand(1,size(X,2))];
res=Y'-X'*((X*X')\(X*Y'));
RSS_withnoise=sum(res.^2)



